<?php

/**
 * @file
 * Access control related functions for the Subnodes by taxonomy module.
 */

/**
 * Check node access for child and parent nodes pair.
 */
function _subnodes_by_taxonomy_node_access($op, $parent_node = NULL, $child_node = NULL) {
  static $access = array();

  if (!isset($access[$op])) {
    $access[$op] = array();
  }

  // Check for create access. See _node_add_access() in node.module.
  if ($op == 'create') {
    $nodetype = str_replace('-', '_', $child_node);
    if (!isset($access[$op][$nodetype])) {
      $access[$op][$nodetype] = node_hook($nodetype, 'form') && node_access('create', $nodetype);
    }
    return $access[$op][$nodetype];
  }

  // Check for clone access.
  if ($op == 'clone') {
    if (!isset($access[$op][$child_node->vid])) {
      $access[$op][$child_node->vid] = module_exists('clone') && clone_access($child_node);
    }
    return $access[$op][$child_node->vid];
  }

  // Check for translation access.
  if ($op == 'translate') {
    if (!isset($access[$op][$child_node->vid])) {
      $access[$op][$child_node->vid] = module_exists('translation') && _translation_tab_access($child_node);
    }
    return $access[$op][$child_node->vid];
  }

  // Check for relationships access.
  if ($op == 'relationships') {
    if (!isset($access[$op][$child_node->vid])) {
      $access[$op][$child_node->vid] = module_exists('noderelationships') && noderelationships_backref_access($child_node);
    }
    return $access[$op][$child_node->vid];
  }

  // Check for view/edit/delete access.
  if (!isset($access[$op][$parent_node->vid])) {
    $access[$op][$parent_node->vid] = node_access($op, $parent_node);
  }
  if (!isset($access[$op][$child_node->vid])) {
    $access[$op][$child_node->vid] = ($op == 'delete' && !isset($child_node)) || node_access($op, $child_node);
  }
  return $access[$op][$parent_node->vid] && $access[$op][$child_node->vid];
}
