<?php

/**
 * @file
 * Implementation of node_type_form extensions.
 */

/**
 * Implementation of hook_form_alter() for the content type edit form.
 */
function subnodes_by_taxonomy_form_node_type_alter(&$form, $form_state) {
  $nodetype = $form['#node_type']->type;

  // First, find content types that have a group that only contains 2 fields.
  // Next, check one field is of type 'nodereference', the other 'content_taxonomy'.
  $candidates = array();

  // Outer loop searches for groups with 2 fields in them.
  $outer_result = db_query("SELECT gf.type_name, gf.group_name, g.label, COUNT(*) AS fields
    FROM {content_group_fields} gf
      INNER JOIN {content_node_field} nf ON nf.field_name = gf.field_name
      INNER JOIN {content_group} g ON g.group_name = gf.group_name AND g.type_name = gf.type_name
    WHERE gf.type_name <> '%s'
      AND nf.active = 1 AND nf.locked = 0 AND nf.multiple = 0
    GROUP BY gf.type_name, gf.group_name, g.label
    HAVING COUNT(*) = 2", $nodetype);

  while ($group = db_fetch_object($outer_result)) {
    // Inner loop checks the types of the fields.
    $inner_result = db_query("SELECT nf.field_name, nf.type, fi.label
      FROM {content_group_fields} gf
        INNER JOIN {content_node_field} nf ON nf.field_name = gf.field_name
        INNER JOIN {content_node_field_instance} fi ON fi.field_name = gf.field_name
      WHERE gf.type_name = '%s'
        AND gf.group_name = '%s'", $group->type_name, $group->group_name);

    $group->fields = array();
    while ($field = db_fetch_object($inner_result)) {
      // Ignore groups where one field is not of the required types.
      if ($field->type != 'nodereference' && $field->type != 'content_taxonomy') {
        break;
      }
      if (isset($group->fields[$field->type])) {
        break;
      }
      $group->fields[$field->type] = $field;
    }

    // Ignore groups where both fields are of the required types.
    if (count($group->fields) == 2) {
      if (!isset($candidates[$group->type_name])) {
        $candidates[$group->type_name] = array();
      }
      $candidates[$group->type_name][$group->group_name] = array(
        'label' => $group->label,
        'fields' => $group->fields,
      );
    }
  }

  // Build the form fieldset.
  $form['subnodes_by_taxonomy'] = array(
    '#type' => 'fieldset',
    '#title' => t('Subnodes by taxonomy settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => '<p>'. t('These options allow you to configure nodes of this content type to become parents of nodes of another type. It is required that child nodes are configured with a fieldgroup that containts two fields: a node reference that will hold the reference to the parent and a taxonomy field that will hold the term that assigns meaning to the relation.') .'</p>',
  );
  if (empty($candidates)) {
    $form['subnodes_by_taxonomy']['#description'] .= '<div class="error">'. t('No content types with the required criteria have been found.') .'</div>';
  }
  $options = array('' => t('-- None --'));
  foreach ($candidates as $type => $groups) {
    $type_name = node_get_types('name', $type);
    if (module_exists('i18ncontent')) {
      $type_name = tt("nodetype:type:$type:name", $type_name);
    }
    $choices = array();
    foreach ($groups as $group_name => $group_info) {
      $choices[$type .'|'. $group_name] = t($group_info['label']) .' ('. t($group_info['fields']['nodereference']->label) .' + '. t($group_info['fields']['content_taxonomy']->label) .')';
    }
    $options[$type_name] = $choices;
  }
  $relation = subnodes_by_taxonomy_get_relation($nodetype, FALSE);
  $form['subnodes_by_taxonomy']['subnodes_by_taxonomy_child_group'] = array(
    '#type' => 'select',
    '#title' => t('Child field group'),
    '#options' => $options,
    '#default_value' => ($relation !== FALSE && !empty($relation->child_type) ? $relation->child_type .'|'. $relation->group_name : NULL),
    '#description' => t('Select the field group that will be used to store parentship information on the child nodes.'),
  );

  $form['subnodes_by_taxonomy']['subnodes_by_taxonomy_redirect_child'] = array(
    '#type' => 'checkbox',
    '#title' => t('Redirect child nodes to parent'),
    '#default_value' => ($relation !== FALSE && isset($relation->settings['redirect_child_to_parent']) ? $relation->settings['redirect_child_to_parent'] : 0),
    '#description' => t('Check this option when you wish to redirect all node view requests to child nodes to their corresponding parent.'),
  );

  // Prepend our submit handler.
  if (!isset($form['#submit'])) {
    $form['#submit'] = array();
  }
  array_unshift($form['#submit'], 'subnodes_by_taxonomy_form_node_type_submit');
}

/**
 * Submit handler for our node type form extensions.
 */
function subnodes_by_taxonomy_form_node_type_submit($form, &$form_state) {
  $form_values = $form_state['values'];
  $new_type = trim($form_values['type']);
  $old_type = (!empty($form_values['old_type']) ? $form_values['old_type'] : $new_type);
  db_query("DELETE FROM {subnodes_by_taxonomy_relations} WHERE parent_type = '%s'", $old_type);
  if ($new_type == $old_type && !empty($form_values['subnodes_by_taxonomy_child_group'])) {
    list($child_type, $group_name) = explode('|', $form_values['subnodes_by_taxonomy_child_group']);
    $settings = array(
      'redirect_child_to_parent' => (isset($form_values['subnodes_by_taxonomy_redirect_child']) ? (int)$form_values['subnodes_by_taxonomy_redirect_child'] : 0),
    );
    db_query("INSERT INTO {subnodes_by_taxonomy_relations} (parent_type, child_type, group_name, settings) VALUES ('%s', '%s', '%s', '%s')", array($old_type, $child_type, $group_name, serialize($settings)));
  }
  // Remove our elements from the form.
  unset($form_state['values']['subnodes_by_taxonomy_child_group'], $form_state['values']['subnodes_by_taxonomy_redirect_child']);
}
