
(function ($) {

/**
 * Subnodes by taxonomy behavior.
 */
Drupal.behaviors.subnodesByTaxonomy = function(context) {
  var self = Drupal.subnodesByTaxonomy;

  // Capture node title changes.
  $('#edit-title:not(.subnodes-by-taxonomy-processed)', context).addClass('subnodes-by-taxonomy-processed').change(function() {
    var title = this.value;
    $('.subnodes-by-taxonomy-child-caption h2').hide('fast', function() {
      $('span', this).text(title);
      $(this).show('fast');
    });
  });

  // Attach tabs behavior.
  $('.subnodes-by-taxonomy-tabset-nav:not(.subnodes-by-taxonomy-processed)', context).addClass('subnodes-by-taxonomy-processed').each(function() {
    var nodeTabs = new self.nodeTabs(this);
  });
};

/**
 * Private namespace for local methods.
 */
Drupal.subnodesByTaxonomy = Drupal.subnodesByTaxonomy || {};

/**
 * Build information about the tabs from a tabset.
 */
Drupal.subnodesByTaxonomy.nodeTabs = function(tabSet) {
  var nodeTabs = this, self = Drupal.subnodesByTaxonomy;
  nodeTabs.busy = false;
  nodeTabs.tabSet = tabSet;
  nodeTabs.selectedTab = null;

  $('a', nodeTabs.tabSet).each(function() {
    var tab = this, selector = $(tab).attr('rel');
    $(tab).attr('loaded', 0).bind('click', function() {
      if (nodeTabs.busy || nodeTabs.selectTab(selector) === false) {
        $(tab).blur();
      }
      return false;
    });
  });

  $(nodeTabs.tabSet).bind('tab.select', function(event, selector, tab) {
    if ($(tab).parents('li:first').hasClass('subnodes-by-taxonomy-tabset-addtab')) {
      nodeTabs.hideStatus();
      self.promptChildTerm($(tab).attr('href'), nodeTabs);
      return false;
    }
    if (selector) {
      nodeTabs.loadTab(selector);
    }
  });

  if (window.location.hash) {
    var selector = window.location.hash.replace(/#/g, '');
    if ($('a[rel="'+ selector +'"]', nodeTabs.tabSet).size()) {
      nodeTabs.selectTab(selector, function() {
        setTimeout(function() {
          window.location.hash = '';
          window.scrollTo(0, $(nodeTabs.tabSet).offset().top - 30);
        }, 500);
      });
    }
  }

  $('.subnodes-by-taxonomy-tabset-addtab:not(.subnodes-by-taxonomy-processed)').addClass('subnodes-by-taxonomy-processed').show();

  nodeTabs.attachDialogs();
};

/**
 * Select a tab.
 */
Drupal.subnodesByTaxonomy.nodeTabs.prototype.selectTab = function(newSelector, callback) {
  var nodeTabs = this, oldSelector = nodeTabs.selectedTab;
  if ($(nodeTabs.tabSet).triggerHandler('tab.select', [newSelector, $('a[rel="'+ newSelector +'"]', nodeTabs.tabSet)]) === false) {
    return false;
  }

  nodeTabs.hideStatus();

  if (oldSelector) {
    $('a[rel="'+ oldSelector +'"]', nodeTabs.tabSet).parents('li:first').removeClass('subnodes-by-taxonomy-selected');
    if (oldSelector == newSelector) {
      nodeTabs.selectedTab = null;
      $('#'+ oldSelector).fadeOut('fast', callback || function() {});
      return;
    }
  }
  nodeTabs.selectedTab = newSelector;

  if (newSelector) {
    $('a[rel="'+ newSelector +'"]', nodeTabs.tabSet).parents('li:first').addClass('subnodes-by-taxonomy-selected');
  }

  var animationCallback = callback || function() {
    setTimeout(function() {
      window.scrollTo(0, $(nodeTabs.tabSet).offset().top - 30);
    }, 100);
  };

  if (oldSelector) {
    if (newSelector) {
      $('#'+ oldSelector).fadeOut('fast');
      $('#'+ newSelector).fadeIn('fast', animationCallback);
    }
    else {
      $('#'+ oldSelector).fadeOut('fast', animationCallback);
    }
  }
  else {
    if (newSelector) {
      $('#'+ newSelector).fadeIn('fast', animationCallback);
    }
    else {
      animationCallback();
    }
  }
  return true;
};

/**
 * Load a tab.
 */
Drupal.subnodesByTaxonomy.nodeTabs.prototype.loadTab = function(selector, reload) {
  var nodeTabs = this, self = Drupal.subnodesByTaxonomy;
  var $currentTab = $('a[rel="'+ selector +'"]', nodeTabs.tabSet);
  if ($currentTab.size() <= 0 || ($currentTab.attr('loaded') == 1 && !reload)) {
    return;
  }

  // Start animation; restored during ajax complete().
  nodeTabs.busy = true;
  Drupal.freezeHeight();
  if (!$('.subnodes-by-taxonomy-throbber').size()) {
    $('#'+ selector +' .subnodes-by-taxonomy-child-caption').append('<div class="subnodes-by-taxonomy-throbber"/>');
  }
  var $subnodeTasks = $('#'+ selector +' .subnodes-by-taxonomy-child-caption').find('.subnodes-by-taxonomy-subnode-tasks');
  if ($subnodeTasks.size()) {
    $subnodeTasks.fadeOut('fast');
  }

  var url = $currentTab.attr('href').replace(/^(.*)\/view$/, '$1/ajax');

  $.ajax({
    url: url,
    dataType: 'json',
    type: 'POST',
    success: function(response, status) {
      if (typeof(response) == 'string') {
        response = Drupal.parseJson(response);
      }
      var $wrapper = $('#'+ selector +' .subnodes-by-taxonomy-child-body');
      var $content = $('<div/>').html(response.data);
      $wrapper.empty().append($content);
      if (response.__callbacks) {
        $.each(response.__callbacks, function(i, callback) {
          eval(callback)($content, response);
        });
      }
      else {
        Drupal.attachBehaviors($content);
      }
      $currentTab.attr('loaded', 1);
    },
    complete: function(response, status) {
      if (status == 'error' || status == 'parsererror') {
        alert(Drupal.ahahError(response, url));
      }
      $('.subnodes-by-taxonomy-throbber').remove();
      if ($subnodeTasks.size()) {
        $subnodeTasks.fadeIn('fast', function() {
          Drupal.unfreezeHeight();
          nodeTabs.busy = false;
        });
      }
      else {
        Drupal.unfreezeHeight();
        nodeTabs.busy = false;
      }
    }
  });
};

/**
 * Create a new tab.
 */
Drupal.subnodesByTaxonomy.nodeTabs.prototype.addTab = function(tabInfo) {
  var nodeTabs = this, settings = Drupal.settings.subnodesByTaxonomy;
  if ($('a[rel="'+ tabInfo.selector +'"]', nodeTabs.tabSet).size()) {
    nodeTabs.selectTab(tabInfo.selector);
    return;
  }

  settings.childrenTerms[tabInfo.tid].available = false;

  nodeTabs.busy = true;
  nodeTabs.selectTab(null, function() {
    var $lis = $('li:has(a[href])', nodeTabs.tabSet);
    var $li = $(Drupal.theme('subnodesByTaxonomyTabTemplate', tabInfo.url, tabInfo.label, tabInfo.title));
    $li.children('a').attr({
      rel: tabInfo.selector,
      loaded: 0
    }).bind('click', function() {
      if (nodeTabs.busy || nodeTabs.selectTab(tabInfo.selector) === false) {
        $(this).blur();
      }
      return false;
    });
    $li.insertBefore($lis[tabInfo.index]);

    $(nodeTabs.tabSet).parents('.subnodes-by-taxonomy-tabset-wrapper:first').append($(tabInfo.panel));
    nodeTabs.attachDialogs();
    nodeTabs.selectTab(tabInfo.selector, function() {
      nodeTabs.busy = false;
    });
  });
};

/**
 * Delete the currently selected tab.
 */
Drupal.subnodesByTaxonomy.nodeTabs.prototype.deleteTab = function(tabInfo) {
  var nodeTabs = this, settings = Drupal.settings.subnodesByTaxonomy;

  settings.childrenTerms[tabInfo.tid].available = true;

  nodeTabs.selectTab(null, function() {
    nodeTabs.busy = true;
    $('a[rel="'+ tabInfo.selector +'"]', nodeTabs.tabSet).parents('li:first').fadeOut('fast', function() {
      $(this).remove();
    });
    $('#'+ tabInfo.selector).remove();
    nodeTabs.busy = false;
  });
};

/**
 * Show status messages from Modal Frame responses on top of tabs.
 */
Drupal.subnodesByTaxonomy.nodeTabs.prototype.showStatus = function(statusMessages) {
  var nodeTabs = this;
  $(nodeTabs.tabSet).parents('.subnodes-by-taxonomy-tabset-wrapper:first').each(function() {
    var $wrapper = $(this);
    $wrapper.find('.subnodes-by-taxonomy-tabset-status').remove();
    if (statusMessages) {
      $wrapper.prepend($('<div class="subnodes-by-taxonomy-tabset-status"/>').hide().append(statusMessages));
      $('.subnodes-by-taxonomy-tabset-status', $wrapper).show('fast');
    }
  });
};

/**
 * Hide status messages.
 */
Drupal.subnodesByTaxonomy.nodeTabs.prototype.hideStatus = function() {
  var nodeTabs = this;
  $(nodeTabs.tabSet).parents('.subnodes-by-taxonomy-tabset-wrapper:first').find('.subnodes-by-taxonomy-tabset-status').each(function() {
    var $status = $(this);
    $status.hide('fast', function() {
      $status.remove();
    });
  });
};

/**
 * Attach ui.dialog to node action buttons.
 */
Drupal.subnodesByTaxonomy.nodeTabs.prototype.attachDialogs = function() {
  var nodeTabs = this, self = Drupal.subnodesByTaxonomy;
  $(nodeTabs.tabSet).parents('.subnodes-by-taxonomy-tabset-wrapper:first')
    .find('.subnodes-by-taxonomy-dialog-task:not(.subnodes-by-taxonomy-processed)')
    .addClass('subnodes-by-taxonomy-processed')
    .each(function() {
      var dialogTask = this;
      $(dialogTask).bind('click', function() {
        nodeTabs.hideStatus();
        if ($(dialogTask).hasClass('subnodes-by-taxonomy-dialog-task-addtab')) {
          self.promptChildTerm($(dialogTask).attr('href'), nodeTabs);
        }
        else {
          var width, height, regExp = new RegExp(/^.*subnodes-by-taxonomy-dialog-size\[([0-9]*,[0-9]*)\].*$/);
          if (typeof dialogTask.className == 'string' && regExp.test(dialogTask.className)) {
            var size = dialogTask.className.replace(regExp, '$1').split(',');
            width = parseInt(size[0]);
            height = parseInt(size[1]);
          }
          self.openDialog($(dialogTask).attr('href'), nodeTabs, width, height);
        }
        return false;
      });
    });
};

/**
 * Open the popup dialog.
 */
Drupal.subnodesByTaxonomy.openDialog = function(url, nodeTabs, width, height) {
  var self = this;
  var modalOptions = {
    url: url,
    autoFit: true,
    onSubmit: function(args, statusMessages) {
      if (args && typeof args.operation == 'string') {
        if (args.operation == 'addTab') {
          nodeTabs.addTab(args);
        }
        else if (args.operation == 'deleteTab') {
          nodeTabs.deleteTab(args);
        }
        else if (args.operation == 'updateTab') {
          nodeTabs.loadTab(args.selector, true);
        }
        nodeTabs.showStatus(statusMessages);
      }
    }
  };
  modalOptions.width = (width ? width : ($(window).width() - 30));
  modalOptions.height = (height ? height : ($(window).height() - 30));
  Drupal.modalFrame.open(modalOptions);
};

/**
 * Prompt the user for a children term.
 */
Drupal.subnodesByTaxonomy.promptChildTerm = function(url, nodeTabs) {
  var self = this, settings = Drupal.settings.subnodesByTaxonomy;

  var $promptContainer;
  if ($('#subnodes-by-taxonomy-prompt-container').size()) {
    $promptContainer = $('#subnodes-by-taxonomy-prompt-container');
  }
  else {
    $promptContainer = $('<div id="subnodes-by-taxonomy-prompt-container"/>');
    $('body').append($promptContainer);
  }

  var promptOptions = ['<option value="-1">-- '+ Drupal.t('None') +' --</option>'];
  for (var tid in settings.childrenTerms) {
    var term = settings.childrenTerms[tid];
    if (term.available) {
      promptOptions.push('<option value="'+ tid +'">'+ term.title +'</option>');
    }
  }

  $promptContainer.selectedOption = -1;
  $promptContainer.empty()
    .append($(Drupal.theme('subnodesByTaxonomyPromptForm', settings.childrenTermsTitle, promptOptions)))
    .append($(Drupal.theme('subnodesByTaxonomyPromptOk'))
    .bind('click', function() {
      $promptContainer.selectedOption = $('#prompt-child-term').val();
      $promptContainer.dialog('close');
      return false;
    }))
    .append($(Drupal.theme('subnodesByTaxonomyPromptCancel')).bind('click', function() {
      $promptContainer.dialog('close');
      return false;
    }));

  var destroyDialog = function() {
    $promptContainer.dialog('destroy').remove();
    Drupal.modalFrame.fixPosition($('.modalframe'), false);
    if ($promptContainer.selectedOption > 0) {
      self.openDialog(url +'/'+ $promptContainer.selectedOption, nodeTabs);
    }
  };

  $promptContainer.closing = false;
  $promptContainer.dialog({
    modal: true,
    autoOpen: false,
    closeOnEscape: true,
    resizable: false,
    title: Drupal.t('Choose an option...'),
    dialogClass: 'modalframe',
    open: function() {
      $('.modalframe .ui-dialog-titlebar-close').attr('title', Drupal.t('Close'));
      Drupal.modalFrame.fixPosition($('.modalframe'), true);
    },
    beforeclose: function() {
      if ($.browser.msie) {
        return true;
      }
      if (!$promptContainer.closing) {
        $promptContainer.closing = true;
        $('.modalframe').animate({height: 'hide', opacity: 'hide'}, 200, function() {
          destroyDialog();
        });
      }
      return false;
    },
    close: function() {
      destroyDialog();
    },
    width: 300,
    height: 200
  });

  $promptContainer.dialog('open');
  if (!$.browser.msie) {
    $('.modalframe').hide().animate({height: 'show', opacity: 'show'});
  }
};

/**
 * Theme new tab item.
 */
Drupal.theme.prototype.subnodesByTaxonomyTabTemplate = function(href, label, title) {
  return '<li><a href="'+ href +'" title="'+ title +'">'+ label +'</a></li>';
};

/**
 * Theme the prompt form.
 */
Drupal.theme.prototype.subnodesByTaxonomyPromptForm = function(title, options) {
  var output = '<div class="form-item">';
  output += '<label for="prompt-child-term">'+ title +':</label>';
  output += '<select id="prompt-child-term" name="prompt-child-term">'+ options.join('') +'</select>';
  output += '</div>';
  return output;
};
Drupal.theme.prototype.subnodesByTaxonomyPromptOk = function() {
  return '<input type="submit" class="form-submit" value="'+ Drupal.t('Ok') +'" />';
};
Drupal.theme.prototype.subnodesByTaxonomyPromptCancel = function() {
  return '<input type="submit" class="form-submit" value="'+ Drupal.t('Cancel') +'" />';
};

})(jQuery);
